# Developing analytics.js-video-plaugins

 - [Development Environemnt Setup](#development-setup)
 - [Project Structure](#project-structure)
 - [Code Conventions](#code-conventions)
 - [Exporting Plugins](#exporting-plugins)
 - [Git Commit Guidelines](#git-commit-guidelines)
 - [Running a Local Development Server](#running-a-local-development-server)
 - [Releasing](#releasing)

## Development Setup

To get started, you'll need to have the following tools available in your local development environment:

* [Git](http://git-scm.com/): The [Github Guide to
  Installing Git][https://help.github.com/articles/set-up-git/] is a good source of information.

* [Node.js (v8 or higher)](http://nodejs.org)

* [Yarn](https://yarnpkg.com): We use Yarn to install our Node.js module dependencies. See the detailed [installation instructions](https://yarnpkg.com/en/docs/install#mac-stable).

* [Prettier](https://prettier.io/): All of our code formatting is handled by Prettier. We recommend installing Prettier as an integration into your [editor](https://prettier.io/docs/en/editors.html) of choice.

Once you have these dependencies in place, feel free to clone this repo locally to get started:

```bash
$ git clone https://github.com/segmentio/analytics.js-video-plugins
$ cd analytics.js-plugins
```

## Project Structure

The project is structured as a monorepo where each plugin is independently deployed as it's own npm module out of the `/plugins` folder.

All plugins adhere to the following directory structure:

```
+ plugin-name
  + src
    - index.js
  + tests
    - index.js
  - package.json
  - README.md
  - CHANGELOG.md
```

## Code Conventions

The project uses `prettier` as it's code formatting standard. The project also uses Babel to support the use of modern JS syntax while also supporting browsers back to IE9. 

All plugins are written and exported as JS [Classes](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Classes):

```js
import exportPlugin from '../../../lib/exportPlugin'

class VimeoAnalytics {
  initialize() {
    doSomething()
  }
}

export default exportPlugin(VimeoAnalytics)
```
## Exporting Plugins
Analytics.js Plugins are designed to be imported either as an npm module *or* as a direct embed from a CDN via a `<script>` tag. 

To support the latter, we need to mount the plugins somewhere that they can be accessed globally. In order to avoid polluting the global scope, we add them as members of an object called `plugins` that is attached to the global `window.analytics` object that `analytics.js` exports itself as. In the above example, the `VimeoAnalytics` plugin will be available at `window.analytics.plugins.VimeoAnalytics`.

The `exportPlugin` function helps automate the logic necessary to support both of these import options.

## Git Commit Guidelines
All plugins are carefully versioned to ensure stability. All version changes are logged thoroughly using the [Angular git commit guidelines](https://github.com/angular/angular.js/blob/master/DEVELOPERS.md#-git-commit-guidelines).

The project uses [Commitizen](https://github.com/commitizen/cz-cli) for easy implementation of these standards. 

**To use commitizen, run `yarn commit` in place of `git commit` for ALL commits.**

### Commit Scope
If you're working on a plugin, the [scope](https://github.com/angular/angular.js/blob/master/DEVELOPERS.md#scope) of your commits should exactly match the name of the directory of the plugin the change pertains to (`vimeo`, `youtube`, etc). If you're making a change outside of the `/plugins` directory, please use something simple like `ci`, `webpack`, `yarn`, `babel` etc.

### Breaking Changes
Commitizen provides a prompt to classify a commit as a breaking change. **It is very important that this prompt is adhered to as it will be used to trigger a major version bump in CI**.

## Running a Local Development Server
If you would like to locally simulate the CDN style distribution of a plugin, you can make it accessible behind a localhost server by running `yarn cdn`. All plugins will now be accessible at the following URL: `http://localhost:9002/{{plugin-directory}}/dist/{{plugin-directory}}.js`

You can now embed each plugin via a `script` tag on a locally running website. Here's an example using the Vimeo plugin:

```html
<script src="http://localhost:9002/vimeo/dist/vimeo.js"></script>
```

## Releasing
Please see separate file RELEASING.md for instructions on releasing your changes once they've been merged into the master branch
